#include <libopencm3/cm3/common.h>
#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/gpio.h>

/*

 *********************************************************
 * Follow hints from https://www.kaltpost.de/?page_id=412
 *********************************************************
 * Timers 6 and 7 are mainly used for DAC, the timers 1 and 8 are advanced timers also providing hal-sensor and quadrature support.
 * The timers 2, 3, 4, and 5 are general purpose timers.
 * For PWM generation, the timers 2 to 5 are good enough.
 * Each of the timers 2 to 5 has four output channels. EX: Timer2 has output TIM2_CH1=>A0,TIM2_CH2=>A1,TIM2_CH3=>A2,TIM2_CH4=>A3.
 * Clock frequency for the timers 2 to 5 is by default 72Mhz.
 * Each timer has a counter register CNT. This register is increased by one with each clock tick it receives.
 * Count frequency in CNT could be modified by a prescaling factor in the PSC register.
 * The period for the PWM is defined by the value in the ARR register.
 * As soon as CNT has reached ARR, CNT is reset to zero and the period of the PWM is over.
 * To calculate the correct value for the ARR register, we divide the result of the prescaled frequency by the intended frequency for our period.

 * All the the registers CNT, PSC and ARR apply to the whole timer and thus are used on all four output channels.

 * To state how log the duty cycle should be, there is the CCRx register for each output channel.
 * As soon as CNT has reached the value in CCRx, the output is driven low for the rest of the period.

 *********************************************************
 * SERVO MOTOR SG90
 * 20ms PWM Period (PWM_Frequency = 50 Hz) for 1ms-2ms of duty cicle
 * Position "0" (1.5 ms pulse) is middle, "90" (~2ms pulse),
 * is all the way to the right, "-90" (~1ms pulse) is all the way to the left

 */

#include "servo.h"

void delay(int d);
void delay(int d)
{

     int i;

     for(i = 0; i < d; i++) {
          __asm("nop");
     }
}

int main( void )
{
	/* set STM32 to 72 MHz */
	rcc_clock_setup_in_hse_8mhz_out_72mhz();

	/* enable GPIOC clock */
	rcc_peripheral_enable_clock(&RCC_APB2ENR, RCC_APB2ENR_IOPCEN);

	/* set GPIO13 at PORTC (led) to 'output alternate function push-pull'. */
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_50_MHZ, GPIO_CNF_OUTPUT_PUSHPULL, GPIO13);

	servo_init();

	while(1) {
		servo_set_position(SERVO_CH1, SERVO_MIN);
		delay(5000000);
		gpio_toggle(GPIOC, GPIO13);

		servo_set_position(SERVO_CH1, SERVO_NULL);
		delay(5000000);
		gpio_toggle(GPIOC, GPIO13);

		servo_set_position(SERVO_CH1, SERVO_MAX);
		delay(5000000);
		gpio_toggle(GPIOC, GPIO13);

		servo_set_position(SERVO_CH1, SERVO_MIN);
		delay(5000000);
		gpio_toggle(GPIOC, GPIO13);
		servo_set_position(SERVO_CH1, SERVO_MAX);
		delay(5000000);
		gpio_toggle(GPIOC, GPIO13);

		servo_set_position(SERVO_CH1, SERVO_MIN);
		delay(2500000);
		gpio_toggle(GPIOC, GPIO13);
		servo_set_position(SERVO_CH1, SERVO_MAX);
		delay(2500000);
		gpio_toggle(GPIOC, GPIO13);

		servo_set_position(SERVO_CH1, SERVO_NULL);
		delay(1000000);
		gpio_toggle(GPIOC, GPIO13);
	}

	return 0;
}
